﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Modbus
{
    class ModbusASCII
    {
        /* Function builds message from binary data and writes it as hex in ascii (big endian) */
        public static byte[] BuildMessage(byte[] data)
        {
            byte[] returnData = new byte[5 + data.Length * 2];

            returnData[0] = (byte)':';
            uint offset = BinaryToHex(data, ref returnData, 1); // Writing PDU
            offset = ModbusASCII.GetCheckSum(data, 0, (uint)data.Length, ref returnData, offset);
            returnData[offset++] = 0x0D;
            returnData[offset++] = 0x0A;

            return returnData;
        }

        /* Function converts binary data to hex written in ascii (big endian) */
        private static uint BinaryToHex(byte[] srcData, ref byte[] destData, uint destOffset)
        {
            for (uint i = 0; i < srcData.Length; ++i)
            {
                destData[destOffset++] = ModbusASCII.getASCIICharFromInt((srcData[i] & 0xf0u) >> 4);
                destData[destOffset++] = ModbusASCII.getASCIICharFromInt(srcData[i] & 0x0fu);
            }
            return destOffset;
        }

        /* Function calcs LRC for asci data */
        public static uint GetCheckSum(byte[] data, uint dataOffset, uint dataSize, ref byte[] dest, uint destOffset)
        {
            byte lrc = 0;

            for (uint i = dataOffset; i < dataSize; ++i)
            {
                lrc += data[i];
            }
            lrc = (byte)-lrc;

            dest[destOffset++] = getASCIICharFromInt((lrc & 0xf0u) >> 4);
            dest[destOffset++] = getASCIICharFromInt(lrc & 0x0fu);
            return destOffset;
        }

        /* Function gets hex ascii value from integer */
        public static byte getASCIICharFromInt(uint val)
        {
            if (val < 10)
            {
                return (byte)('0' + val);
            }
            else if (val < 16)
            {
                return (byte)('A' + val - 10);
            }
            else
            {
                return 0;
            }
        }
        /* Function gets integer from hex ascii value */
        public static byte getHalfByteFromASCIIChar(byte ascii)
        {
            if (ascii >= 'A' && ascii <= 'F')
            {
                return (byte)(ascii - 'A' + 10);
            }
            else if (ascii >= '0' && ascii <= '9')
            {
                return (byte)(ascii - '0');
            }
            else
            {
                return 0;
            }
        }
    }
}
