﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;
using System.Windows.Forms;
using System.Threading;

namespace Modbus
{
    interface IDataReceiveListener
    {
        void DataReceivedEvent(byte[] data);
    }

    class SerialConnector 
    {
        SerialPort sp = null;
        string lastStatus = null;

        bool bReceiving = false;
        Thread transmissionThread = null;
        IDataReceiveListener receiveHandler = null;

        public SerialConnector(IDataReceiveListener handler)
        {
            sp = new SerialPort();
            sp.ReadTimeout = 1000;
            sp.WriteTimeout = 1000;

            receiveHandler = handler;
        }

        public string GetLastStatus()
        {
            return lastStatus;
        }

        public bool Open(string portName, int baudRate, int wordLength, StopBits stopBits, Parity parity)
        {
            if (!sp.IsOpen)
            {
                sp.PortName = portName;
                sp.BaudRate = baudRate;
                sp.DataBits = wordLength;
                sp.Parity = parity;
                sp.StopBits = stopBits;
                sp.Encoding = Encoding.BigEndianUnicode;

                try
                {
                    sp.Open();
                }
                catch (Exception err)
                {
                    lastStatus = "Blad przy otwieraniu portu " + portName + ": " + err.Message;
                    return false;
                }

                bReceiving = true;
                transmissionThread = new Thread(() => { Receive(); });
                transmissionThread.Start();

                lastStatus = portName + " zostal otwarty";
                return true;
            }
            else
            {
                lastStatus = portName + " jest juz otwarty";
                return false;
            }
        }

        public bool Close()
        {
            if (sp.IsOpen)
            {
                try
                {
                    StopReceveing();

                    sp.Close();
                }
                catch (Exception err)
                {
                    lastStatus = "Blad przy zamykaniu portu " + sp.PortName + ": " + err.Message;
                    return false;
                }
                lastStatus = sp.PortName + " zostal zamkniety";
                return true;
            }
            else
            {
                lastStatus = sp.PortName + " jest juz zamkniety";
                return false;
            }
        }

        public void StopReceveing()
        {
            if (transmissionThread != null)
            {
                bReceiving = false;
                while (transmissionThread.ThreadState == ThreadState.Running) ;
            }
        }

        public void Receive()
        {
            byte[] data;

            while (bReceiving)
            {
                if (sp.BytesToRead > 0)
                {
                    data = new byte[sp.BytesToRead];
                    sp.Read(data, 0, data.Length);
                    receiveHandler.DataReceivedEvent(data);
                }

                Thread.Sleep(50);
            }
        }

        public void SendData(byte[] data)
        {
            Thread.BeginCriticalRegion();

            sp.Write(data, 0, data.Length);

            Thread.EndCriticalRegion();
        }
    }
}
